package de.sayk;

import java.net.URI;
import java.net.URL;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Scanner;
import java.util.jar.JarFile;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import com.sun.jersey.api.container.httpserver.HttpServerFactory;
import com.sun.net.httpserver.HttpServer;

import de.sayk.data.Database;
import de.sayk.view.MesMainView;

public class MesApp {

	private static Logger log = Logger.getLogger(MesApp.class.getName());

	private enum MesMode {
		SERVER, CLIENT, ALL;
	}

	private static MesMode mod = MesMode.ALL;

	public static String title = "sfmMES";
	public static String version = "1.0.BETA";
	public static String vendor = "all rights reserved by Stefan Sayk, ${TODAY}";

	private static MesServer msrv;
	private static Thread serverThread;
	public static MesService ms;

	public static void main(String[] args) {

		PropertyConfigurator.configure("log4j.properties");
		log.debug("MES gestartet...");

		try {
			URL url = MesApp.class.getResource("");
			URI uri = new URI(url.getPath().split("!")[0]);
			JarFile myJar = new JarFile(uri.getPath());
			title = myJar.getManifest().getMainAttributes().getValue("Implementation-Title");
			version = myJar.getManifest().getMainAttributes().getValue("Implementation-Version");
			vendor = myJar.getManifest().getMainAttributes().getValue("Implementation-Vendor");

			log.debug("Title: " + title);
			log.debug("Version: " + version);
			log.debug("Vendor: " + vendor);
		} catch (Exception ex) {
		}

		log.debug("check mes mode parameter...");
		for (String s : args) {
			if (s.equalsIgnoreCase("-server")) {
				log.debug("mes server mode");
				mod = MesMode.SERVER;
				break;
			}
			if (s.equalsIgnoreCase("-client")) {
				log.debug("mes client mode");
				mod = MesMode.CLIENT;
				break;
			}
		}

		HttpServer restServer = null;
		if (mod == MesMode.ALL || mod == MesMode.SERVER) {

			log.debug("setup database if not exists...");
			Database.initDb();

			log.debug("rest service starten...");
			try {
				restServer = HttpServerFactory.create("http://localhost:8042/mes");
				restServer.start();
			} catch (Exception e) {
				log.error("Problem starting the rest server...", e);
				Utils.showErrorOptionPane(
						"Konnte den REST Server nicht starten! Bitte stellen Sie sicher, dass nicht schon ein MES auf diesem PC aktiv ist.\n"
								+ e.getLocalizedMessage());
			}

			log.debug("mes service starten...");

			try {

				msrv = new MesServer();

				Registry rmiRegistry = LocateRegistry.createRegistry(2019);
				MesService mesService = (MesService) msrv;
				rmiRegistry.bind("MesService", mesService);

				msrv.setup();
				serverThread = new Thread(msrv);
				serverThread.start();

			} catch (Exception e) {
				e.printStackTrace();
			}

			if (mod == MesMode.ALL) {
				try {
					ms = msrv;
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

		}

		if (mod == MesMode.CLIENT) {
			if (System.getSecurityManager() == null)
				System.setSecurityManager(new RMISecurityManager());
			try {
				ms = (MesService) Naming.lookup("//localhost:2019/MesService");

			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}

		if (mod == MesMode.ALL || mod == MesMode.CLIENT) {
			javafx.application.Application.launch(MesMainView.class);
			log.debug("Oberflche wurde beendet.");
		}

		if (mod == MesMode.SERVER) {
			String cmd = "";
			System.out.println("mes server is running type 'shutdown' for shutdown ;-)");
			while (!cmd.equals("shutdown")) {
				Scanner sc = new Scanner(System.in);
				cmd = sc.next();
			}
		}

		if (mod == MesMode.ALL || mod == MesMode.SERVER)

		{
			log.debug("stoppe services...");
			msrv.shutdown();
			if (restServer != null)
				restServer.stop(0);
		}
		log.debug("bye, bye.");

		System.exit(0);

	}

}