package de.sayk.view;

import de.sayk.Utils;
import de.sayk.data.objects.Order;
import de.sayk.data.objects.OrderState;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.effect.DropShadow;
import javafx.scene.image.Image;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.TextAlignment;

public class ActiveOrderPane extends Pane{

	final static int HEIGHT = 242;
	final static int WIDHT = 470;

	Order order = null;
	boolean isActive;

	Canvas backgCanv = new Canvas(WIDHT, HEIGHT);
	Canvas canv = new Canvas(WIDHT, HEIGHT);
	GraphicsContext gcb = backgCanv.getGraphicsContext2D();
	GraphicsContext gc = canv.getGraphicsContext2D();

	public ActiveOrderPane() {
		super();
		this.getChildren().add(backgCanv);
		this.getChildren().add(canv);
		canv.toFront();
		repaint();
	}
	

	public void startProcess() {
		isActive = true;
		repaint();
	}

	public void endProcess() {
		isActive = false;
		repaint();
	}

	public void setOrder(Order o) {
		order = o;
		repaint();
	}

	public Order getOrder() {
		return order;
	}

	public boolean isActiveOrder() {
		return order != null;
	}

	private void repaint() {

		// paint Background
		gcb.setLineWidth(0);
		gcb.setFill(MyColor.CONTROL_1);
		gcb.fillRect(0, 0, WIDHT, HEIGHT);

		gc.clearRect(0, 0, WIDHT, HEIGHT);

		if (order == null) {

			gc.setFill(MyColor.CONTROL_2);
			gc.setTextAlign(TextAlignment.CENTER);
			gc.setFont(Font.font("Arial", FontWeight.BOLD, 40));
			gc.fillText("kein aktiver Auftrag", WIDHT / 2, HEIGHT / 2);

			// gc.applyEffect(new DropShadow(10, 10, 0, Color.BLACK));

		} else {

			gc.setFill(MyColor.CONTROL_2);
			gc.fillRect(50, 80, WIDHT - 50, HEIGHT - 80);

			for (int i = 0; i < 4; i++) {
				gc.setFill(MyColor.CONTROL_1);
				gc.fillRect(50, (50 * i) + 80, WIDHT - 50, 1);

			}

			String imagePath = "de/sayk/res/Pfeil.png";
			Image image = new Image(imagePath);
			gc.drawImage(image, 7, 135, 35, 45);

			// paint Entries
			if (isActive) {
				gc.setFill(MyColor.HIGH_LIGHT);
				gc.fillRect(50, 130, WIDHT - 50, 50);
			}

			gc.setFill(Color.web(order.getWebColor()));
			gc.fillOval(5, 17, 40, 40);

			gc.setFill(Utils.getForeGroundColorBasedOnBGBrightness(Color.web(order.getWebColor())));

			gc.setTextAlign(TextAlignment.CENTER);
			gc.setFont(Font.font("Arial", FontWeight.BOLD, 20));
			gc.fillText(order.getPart().getRfid() + "", 25, 45);

			gc.setFill(MyColor.FONT);// gelb
			gc.setTextAlign(TextAlignment.LEFT);
			gc.setFont(Font.font("Arial", FontWeight.BOLD, 20));
			gc.fillText(
					order.getProduct().getName() + " (" + order.getId() + ", " + order.getCostumer().getContact() + ")",
					55, 30);
			gc.fillText(order.getVariant().getName(), 55, 55);

			gc.setFill(MyColor.CONTROL_1);
			gc.setTextAlign(TextAlignment.LEFT);
			gc.setFont(Font.font("Arial", FontWeight.NORMAL, 20));
			if (order.getActStep() > 0) {
				gc.fillText(order.getVariant().getWorkflow().get(order.getActStep() - 1).getDesc(), 70, 110);
			}

			gc.setFill(MyColor.FONT);
			int maxSteps = order.getVariant().getWorkflow().size();
			for (int i = 0; i < maxSteps - order.getActStep(); i++) {
				gc.fillText(order.getVariant().getWorkflow().get(order.getActStep() + i).getDesc(), 70, 160 + (i * 50));
			}

		}

	}

}
