package de.sayk.view;

import java.awt.Desktop;
import java.io.File;
import java.io.Serializable;
import java.io.StringWriter;
import java.net.URI;
import java.rmi.RemoteException;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import de.sayk.MesApp;
import de.sayk.PartListener;
import de.sayk.MesTestClient;
import de.sayk.OrderListener;
import de.sayk.ServerResponse;
import de.sayk.Utils;
import de.sayk.data.BigData;
import de.sayk.data.Database;
import de.sayk.data.objects.Order;
import de.sayk.data.objects.OrderState;
import de.sayk.data.objects.Part;
import de.sayk.data.objects.PartState;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.geometry.VPos;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.control.CheckMenuItem;
import javafx.scene.control.Label;
import javafx.scene.control.Menu;
import javafx.scene.control.MenuBar;
import javafx.scene.control.MenuItem;
import javafx.scene.control.SeparatorMenuItem;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.RowConstraints;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.stage.Stage;

public class MesMainView extends Application {
	private static Logger log = Logger.getLogger(MesMainView.class.getName());

	private static Menu menuData = new Menu("Lernsituationen");
	private static Menu menuEdit = new Menu("Bearbeiten");
	private static OrderView planedOrder = new OrderView();
	private static OrderView finishedOrder = new OrderView();

	final HBox hb = new HBox();
	MesTestClient mtc = new MesTestClient();

	static class OrderUpdateListener implements OrderListener, Serializable {

		private static final long serialVersionUID = -444896542311109163L;

		@Override
		public void updateOrder(Order o) {
			Platform.runLater(new Runnable() {

				@Override
				public void run() {
					ActiveOrderView.setOrder(o);
				}
			});

		}

		@Override
		public void startOrder(Order o) {
			Platform.runLater(new Runnable() {

				@Override
				public void run() {
					ActiveOrderView.startOrder(o);
					planedOrder.remove(o);
				}
			});

		}

		@Override
		public void startProcess(Order o) {
			Platform.runLater(new Runnable() {

				@Override
				public void run() {
					ActiveOrderView.setOrder(o);
					ActiveOrderView.startProcess(o);
				}
			});

		}

		@Override
		public void endProcess(Order o) {
			Platform.runLater(new Runnable() {

				@Override
				public void run() {
					ActiveOrderView.setOrder(o);
					ActiveOrderView.endProcess(o);
				}
			});

		}

		@Override
		public void finishOrder(Order o) {
			Platform.runLater(new Runnable() {

				@Override
				public void run() {
					ActiveOrderView.finishOrder(o);;
					finishedOrder.add(o);
				}
			});

		}

	}

	public static void main(String[] args) {
		PropertyConfigurator.configure("log4j.properties");
		launch(args);

	}// 01577 1987513

	public static void addNewOrder(Order o) {
		planedOrder.add(o);
	}

	@Override
	public void start(Stage stage) {

		Image image = new Image("de/sayk/res/Icon.png");
		stage.getIcons().add(image);

		Scene scene = new Scene(new Group());
		stage.setTitle("sfmMES Manufactoring Execution System");
		stage.setWidth(995);
		stage.setHeight(675);

		scene.setFill(Color.web("3faf6c")); // hell gruen
		scene.setFill(Color.LIGHTSLATEGREY); // hell grau
		scene.setFill(Color.DARKSLATEGRAY); // dunkel grau
		scene.setFill(MyColor.BACKGROUND);

		MenuBar menuBar = new MenuBar();

		Menu menuMes = new Menu("MES");
		MenuItem newOrder = new MenuItem("neuen Auftrag anlegen");
		newOrder.setOnAction(new EventHandler<ActionEvent>() {
			public void handle(ActionEvent t) {
				if (Desktop.isDesktopSupported()) {
					try {
						Desktop.getDesktop().browse(new URI("http://127.0.0.1:8042/mes/neworder"));
					} catch (Exception e) {
						log.error("problem show the new order page:", e);
					}
				}
			}
		});
		menuMes.getItems().addAll(newOrder);

		MenuItem nextStep = new MenuItem("nchster Produktionsschritt");
		nextStep.setOnAction(new EventHandler<ActionEvent>() {
			public void handle(ActionEvent t) {
				try {
					MesApp.ms.nextProductionStep(52); //TODO: active rfid tab
				} catch (RemoteException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});
		menuMes.getItems().addAll(nextStep);

		CheckMenuItem testIt = new CheckMenuItem("Teste MES Funktionen mit QS Client");
		testIt.setOnAction(new EventHandler<ActionEvent>() {

			public void handle(ActionEvent t) {
				if (testIt.isSelected()) {
					mtc = new MesTestClient();
					mtc.start();
				} else {
					mtc.interruptAll();
				}
			}
		});
		menuMes.getItems().addAll(testIt);

		MenuItem info = new MenuItem("Info");
		info.setOnAction(new EventHandler<ActionEvent>() {

			public void handle(ActionEvent t) {

				StringWriter sw = new StringWriter();

				sw.write("Version " + MesApp.version + "\n");
				sw.write("Copyright (C) 2019 Stefan Sayk\n\n");

				sw.write("This program is free software: you can redistribute it and/or modify ");
				sw.write("it under the terms of the GNU General Public License as published by ");
				sw.write("the Free Software Foundation, either version 3 of the License, or");
				sw.write("(at your option) any later version.\n");

				sw.write("This program is distributed in the hope that it will be useful, ");
				sw.write("but WITHOUT ANY WARRANTY; without even the implied warranty of ");
				sw.write("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the ");
				sw.write("GNU General Public License for more details.\n");

				sw.write("You should have received a copy of the GNU General Public License ");
				sw.write("along with this program.  If not, see <http://www.gnu.org/licenses/>.\n");

				Utils.showInfoOptionPane(sw.toString());
			}
		});
		menuMes.getItems().addAll(info);

		menuBar.getMenus().addAll(menuData, menuEdit, menuMes);

		try {
			MesApp.ms.addOrderListener(new OrderUpdateListener());
			log.debug("OrderListener geadded");
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		GridPane grid = new GridPane();
		grid.setHgap(10);
		grid.setVgap(10);
		grid.setPadding(new Insets(5, 10, 0, 10));

		ColumnConstraints cc = new ColumnConstraints(470, 470, 470);
		grid.getColumnConstraints().addAll(cc, new ColumnConstraints(), cc);

		RowConstraints rc = new RowConstraints(100, // minimale Hhe
				100, // bevorzugte Hhe
				100, // maximale Hhe
				Priority.ALWAYS, // vertikale Ausdehnung
				VPos.BOTTOM, // vertikale Anordnung
				true); // vertikale Fllung

		grid.getRowConstraints().addAll(new RowConstraints(), rc, new RowConstraints(), new RowConstraints(),
				new RowConstraints(), rc);

		Image img = new Image("de/sayk/res/OerSfmSoftware.png");

		ImageView iv = new ImageView();
		iv.setImage(img);

		iv.setFitHeight(140);
		iv.setFitWidth(470);

		HBox logo = new HBox();
		logo.setAlignment(Pos.BOTTOM_CENTER);
		logo.getChildren().add(iv);

		grid.add(logo, 0, 0, 1, 2);

		grid.add(getTop1("Lager"), 0, 2);
		grid.add(getTop1("aktive Maschinen"), 0, 4);

		grid.add(getTop1("Auftrge"), 2, 0);
		grid.add(getTop2("geplant"), 2, 0);
		grid.add(getTop2("aktiv"), 2, 2);
		grid.add(getTop2("fertig"), 2, 4);

		grid.add(ClientView.getView(), 0, 5);
		grid.add(StoreView.getView(), 0, 3);

		grid.add(planedOrder.getView(), 2, 1);
		grid.add(ActiveOrderView.getView(), 2, 3);
		grid.add(finishedOrder.getView(), 2, 5, 1, 1);
		// grid.add(table, 2, 9);

		BorderPane border = new BorderPane();
		border.setTop(menuBar);
		border.setCenter(grid);

		((Group) scene.getRoot()).getChildren().addAll(border);

		stage.setScene(scene);
		stage.show();

		refresh();

	}

	public static void refresh() {

		// init store
		try {
			MesApp.ms.refreshAllPartsInStore();

			planedOrder.setOrders(MesApp.ms.getAllOrdersByState(OrderState.PLANED));
			finishedOrder.setOrders(MesApp.ms.getAllOrdersByState(OrderState.FINISH));

		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		planedOrder.refresh();
		finishedOrder.refresh();

		// SAY
		//ActiveOrderView.setOrder(null);

		File[] files = new File("sql").listFiles();

		menuData.getItems().clear();

		for (File file : files) {
			if (file.isFile() && file.getName().endsWith(".sql")) {
				MenuItem menuItem = new MenuItem(file.getName());
				menuItem.setOnAction((ActionEvent t) -> {
					Database.setup("sql/" + menuItem.getText());

					if (menuItem.getText().equals("Data Intelligence.sql"))
						BigData.addProductionData();

					refresh();
				});
				menuData.getItems().addAll(menuItem);
			}
		}

		menuEdit.getItems().clear();

		for (File file : files) {
			if (file.isFile() && file.getName().endsWith(".sql")) {
				MenuItem menuItem = new MenuItem(file.getName());
				menuItem.setOnAction((ActionEvent t) -> {
					Database.editUserScript("sql/" + menuItem.getText());
				});
				menuEdit.getItems().addAll(menuItem);
			}
		}

		menuEdit.getItems().addAll(new SeparatorMenuItem());

		MenuItem refresh = new MenuItem("Aktualisieren");
		refresh.setOnAction(new EventHandler<ActionEvent>() {
			public void handle(ActionEvent t) {
				refresh();
			}
		});
		menuEdit.getItems().addAll(refresh);

	}

	private static HBox getTop1(String text) {
		HBox sublabel = new HBox();
		sublabel.setAlignment(Pos.CENTER_LEFT);
		final Label label = new Label(text);
		label.setFont(Font.font("Arial", FontWeight.BOLD, 20));
		label.setTextFill(Color.web("#FEEFDA"));
		label.setTextFill(MyColor.FONT);
		sublabel.getChildren().add(label);
		return sublabel;
	}

	private static HBox getTop2(String text) {
		HBox sublabel = new HBox();
		sublabel.setAlignment(Pos.CENTER_RIGHT);
		final Label label = new Label(text);
		label.setFont(new Font("Arial", 16));
		label.setTextFill(Color.web("#FEEFDA"));
		label.setTextFill(MyColor.FONT);
		sublabel.getChildren().add(label);
		return sublabel;
	}

}