package de.sayk.view;

import java.util.ArrayList;

import de.sayk.data.objects.Order;
import javafx.beans.property.SimpleStringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;

public class OrderView {
	private TableView table = new TableView();
	private ObservableList<OrderEntry> data = FXCollections.observableArrayList();

	public OrderView() {
	}

	public OrderView(ArrayList<Order> orders) {

		this.setOrders(orders);
	}

	public void setOrders(ArrayList<Order> orders) {

		this.data.clear();
		
		for (Order order : orders) {
			OrderEntry oe = new OrderEntry(order.getId() + "", order.getProduct().getName(),
					order.getVariant().getName(), order.getWebColor(), order.getCostumer().getName());
			this.data.add(oe);
		}
	}

	public void remove(Order o) {
		int i;
		boolean isFound = false;
		for (i = 0; i < data.size(); i++) {
			if (data.get(i).getId().equals(o.getId() + "")) {
				isFound = true;
				break;
			}
		}
		if (isFound)
			data.remove(i);
		table.refresh();
	}

	public void refresh() {
		table.refresh();
	}

	public void add(Order o) {
		OrderEntry oe = new OrderEntry(o.getId() + "", o.getProduct().getName(), o.getVariant().getName(),
				o.getWebColor(), o.getCostumer().getName());
		this.data.add(oe);
	}

	public TableView getView() {

		table.setEditable(true);

		TableColumn idCol = new TableColumn("id");
		idCol.setMaxWidth(40);
		idCol.setCellValueFactory(new PropertyValueFactory<OrderEntry, String>("id"));
		TableColumn prodCol = new TableColumn("Produkt");
		prodCol.setMinWidth(100);
		prodCol.setCellValueFactory(new PropertyValueFactory<OrderEntry, String>("product"));
		TableColumn variCol = new TableColumn("Variante");
		variCol.setMinWidth(180);
		variCol.setCellValueFactory(new PropertyValueFactory<OrderEntry, String>("variant"));
		TableColumn colorCol = new TableColumn("Farbe");
		colorCol.setCellValueFactory(new PropertyValueFactory<OrderEntry, String>("color"));
		TableColumn customerCol = new TableColumn("Kunde");
		customerCol.setCellValueFactory(new PropertyValueFactory<OrderEntry, String>("customer"));

		colorCol.setCellFactory(column -> {
			return new TableCell<OrderEntry, String>() {
				protected void updateItem(String item, boolean empty) {
					super.updateItem(item, empty);

					if (item != null && !empty) {
						setStyle("-fx-background-color:" + item);
					}
				}
			};
		});

		table.setItems(data);

		table.getColumns().addAll(idCol, colorCol, prodCol, variCol, customerCol);

		return table;

	}

	public class OrderEntry {

		private SimpleStringProperty id;
		private SimpleStringProperty product;
		private SimpleStringProperty variant;
		private SimpleStringProperty color;
		private SimpleStringProperty customer;

		private OrderEntry(String id, String product, String variant, String color, String customer) {
			this.id = new SimpleStringProperty(id);
			this.product = new SimpleStringProperty(product);
			this.variant = new SimpleStringProperty(variant);
			this.color = new SimpleStringProperty(color);
			this.customer = new SimpleStringProperty(customer);
		}

		public String getId() {
			return id.get();
		}

		public void setId(String id) {
			this.id.set(id);
		}

		public String getProduct() {
			return product.get();
		}

		public void setProduct(String product) {
			this.product.set(product);
		}

		public String getVariant() {
			return variant.get();
		}

		public void setVariant(String variant) {
			this.variant.set(variant);
		}

		public String getColor() {
			return color.get();
		}

		public void setColor(String color) {
			this.color.set(color);
		}

		public String getCustomer() {
			return customer.get();
		}

		public void setCustomer(String customer) {
			this.customer.set(customer);
		}
	}

}
