package de.sayk.view;

import java.io.Serializable;
import java.rmi.RemoteException;

import org.apache.log4j.Logger;

import de.sayk.MesApp;
import de.sayk.PartListener;
import de.sayk.Utils;
import de.sayk.data.objects.Part;
import de.sayk.data.objects.PartState;
import javafx.animation.AnimationTimer;
import javafx.animation.KeyFrame;
import javafx.animation.KeyValue;
import javafx.animation.Timeline;
import javafx.application.Platform;
import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.TextAlignment;
import javafx.util.Duration;

public class StoreView {
	private static Logger log = Logger.getLogger(StoreView.class.getName());

	final static int HEIGHT = 280;
	final static int WIDHT = 470;

	static Canvas storeLayer = new Canvas(WIDHT, HEIGHT);
	static Canvas partsLayer = new Canvas(WIDHT, HEIGHT);

	static Part[] parts = new Part[6];
	static PartAnimation[] aniParts = new PartAnimation[6];

	static class PartUpdateListener implements PartListener, Serializable {

		private static final long serialVersionUID = 3199558289044473990L;

		@Override
		public void updatePart(Part p, int no) throws RemoteException {
			log.debug("part update auf client angestossen...");
			doUpdate(p, no);
		}

	}

	public static void doUpdate(Part p, int no) {

		log.debug("update einplanen");
		Platform.runLater(new Runnable() {

			@Override
			public void run() {
				log.debug("update wird jetzt ausgefuehrt...");
				log.debug("...setPart...");
				setPart(p, no);
				log.debug("...repaint...");
				repaint();
				log.debug("update ist jetzt vollstndig.");
			}
		});

	}

	public static Pane getView() {

		Pane pane = new Pane();
		pane.getChildren().add(storeLayer);
		pane.getChildren().add(partsLayer);
		partsLayer.toFront();
		try {
			MesApp.ms.addPartListener(new PartUpdateListener());
			log.debug("Part Listener geadded");
		} catch (RemoteException e) {
			log.error("Problem adding PartListener", e);
			e.printStackTrace();
		}

		repaint();

		return pane;

	}

	public static Part[] getParts() {
		return parts;
	}

	public static void setPart(Part p, int place) {

		// do some animation
		if (parts[place - 1] != null && p == null)
			aniParts[place - 1].releaseFromStock();
		if (p != null) {
			aniParts[place - 1] = new PartAnimation();
			aniParts[place - 1].part = p;
			aniParts[place - 1].placeIntoStock();
		}

		parts[place - 1] = p;

	}

	private static void repaint() {

		GraphicsContext gc = storeLayer.getGraphicsContext2D();

		gc.setLineWidth(0);
		gc.setFill(Color.web("167B6D"));// Grn
		gc.setFill(MyColor.CONTROL_1);

		gc.fillRect(0, 0, WIDHT, HEIGHT);

		gc.setFill(Color.web("#FEEFDA")); // blasgelb
		gc.setFill(MyColor.CONTROL_2);
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 2; j++) {
				gc.fillOval(i * 150 + 20, j * 150, 130, 130);
				gc.fillRect(i * 150 + 20, j * 215, 130, 65);

			}
		}

		gc = partsLayer.getGraphicsContext2D();
		gc.clearRect(0, 0, WIDHT, WIDHT);

		gc.setLineWidth(0);

		gc.setTextAlign(TextAlignment.CENTER);
		gc.setFont(Font.font("Arial", FontWeight.BOLD, 36));

		int place = 0;

		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 2; j++) {

				if (aniParts[place] != null) {

					gc.setFill(Color.web(aniParts[place].part.getWebColor()));
					double v = aniParts[place].visible.doubleValue();
					double yPos = (j == 0) ? 1.25 * v - 120 : 280 - (v * 1.25);

					gc.fillOval(i * 150 + 25, yPos, 120, 120);

					gc.setFill(
							Utils.getForeGroundColorBasedOnBGBrightness(Color.web(aniParts[place].part.getWebColor())));

					gc.setTextAlign(TextAlignment.CENTER);
					gc.setFont(Font.font("Arial", FontWeight.BOLD, 36));
					gc.fillText(aniParts[place].part.getRfid() + "", i * 150 + 85, yPos + 70);
					if (aniParts[place].part.getPartState() == PartState.RAW) {
						gc.setFont(Font.font("Arial", FontWeight.NORMAL, 20));
						gc.fillText("ROH", i * 150 + 85, yPos + 95);
					} else {
						gc.setFont(Font.font("Arial", FontWeight.NORMAL, 20));
						gc.fillText("FERTIG", i * 150 + 85, yPos + 95);

					}
				}

				gc.setTextAlign(TextAlignment.LEFT);
				gc.setFont(Font.font("Arial", FontWeight.NORMAL, 18));
				gc.setFill(MyColor.FONT_SMALL);
				gc.fillText(place + 1 + "", i * 150 + 25, j * 255 + 15);

				place++;
			}
		}

	}

	static class PartAnimation {
		public DoubleProperty visible = new SimpleDoubleProperty();
		public Part part;

		public void releaseFromStock() {

			Timeline timeline = new Timeline(new KeyFrame(Duration.seconds(0), new KeyValue(visible, 100)),
					new KeyFrame(Duration.seconds(3), new KeyValue(visible, 0)));

			AnimationTimer timer = new AnimationTimer() {
				@Override
				public void handle(long now) {
					repaint();
					if (visible.intValue() == 0)
						stop();
				}
			};

			timer.start();
			timeline.playFromStart();

		}

		public void placeIntoStock() {
			Timeline timeline = new Timeline(new KeyFrame(Duration.seconds(0), new KeyValue(visible, 0)),
					new KeyFrame(Duration.seconds(3), new KeyValue(visible, 100)));

			AnimationTimer timer = new AnimationTimer() {
				@Override
				public void handle(long now) {
					repaint();
					if (visible.intValue() == 100)
						stop();
				}
			};

			timer.start();
			timeline.playFromStart();

		}
	}

}
